import 'package:stateful_provider/model/process_error.dart';
import 'package:uuid/uuid.dart';

import 'task_provider.dart';

class ProcessStatusProvider extends TaskProvider {
  factory ProcessStatusProvider() => _singleton;

  static final ProcessStatusProvider _singleton =
      ProcessStatusProvider._internal();

  ProcessStatusProvider._internal();

  Set<ProcessError> _exceptions = {};


  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded();
  }

  bool _isProcessing = false;

  void _rebuildIfNeeded() {
    final current = isProcessing;
    if (current == _isProcessing) return;
    _isProcessing = current;
    notifyListeners();
  }

  bool _handlingError = false;

  bool get hasError => _exceptions.isNotEmpty;
  ProcessError currentError() {
    _handlingError = true;
    return _exceptions.first;
  }

  void catchError(ProcessError error) {
    _exceptions.add(error);
    if (_handlingError) return;
    notifyListeners();
  }

  void consumeError(ProcessError error) {
    _exceptions.remove(error);
    _handlingError = false;
    notifyListeners();
  }
}
